(function ($, Drupal, drupalSettings) {
  'use strict';
  Drupal.behaviors.tagifyAutocomplete = {
    attach: function attach(context, settings) {
      // see https://github.com/yairEO/tagify#ajax-whitelist
      let elements = $(once('tagify-widget', 'input.tagify-widget', context));

      elements.each(function () {
        // @todo create a setting form in Drupal to have an editable option.
        let input = this,
          tagify = new Tagify(input, {
            dropdown: {
              enabled: parseInt(input.dataset.suggestionsDropdown),
              highlightFirst: true,
              fuzzySearch: !!parseInt(input.dataset.matchOperator),
              maxItems: input.dataset.maxItems ?? Infinity,
              closeOnSelect: true,
            },
            templates: {
              tag: tagTemplate,
            },
            whitelist: [],
            placeholder: parseInt(input.dataset.placeholder),
          }),
          controller;

        function tagTemplate (tagData) {
          const entityIdDiv = parseInt(input.dataset.showEntityId) &&
          tagData.entity_id
            ? `<div id="tagify__tag-items" class="tagify__tag_with-entity-id"><div class='tagify__tag__entity-id-wrap'><span class='tagify__tag-entity-id'>${tagData.entity_id}</span></div><span class='tagify__tag-text'>${tagData.value}</span></div>`
            : `<div id="tagify__tag-items"><span class='tagify__tag-text'>${tagData.value}</span></div>`

          return `<tag title="${tagData.value}"
            contenteditable='false'
            spellcheck='false'
            tabIndex="-1"
            class="tagify__tag ${tagData.class ? tagData.class : ''}"
            ${this.getAttributes(tagData)}>
              <x id="tagify__tag-remove-button" class='tagify__tag__removeBtn' role='button' aria-label='remove tag'></x>
              ${entityIdDiv}
          </tag>`
        }

        // Avoid creating tag when 'Create referenced entities if they don't already exist' is disallowed.
        tagify.settings.enforceWhitelist = !$(this).
          hasClass('tagify--autocreate')
        tagify.settings.skipInvalid = !!$(this).hasClass('tagify--autocreate')
        // Avoid creating tag when the cardinality is 0.
        tagify.settings.maxTags = $(this).hasClass('tagify--limited')
          ? 1
          : Infinity

        // Bind "DragSort" to Tagify's main element and tell
        // it that all the items with the below "selector" are "draggable".
        new DragSort(tagify.DOM.scope, {
          selector: '.' + tagify.settings.classNames.tag,
          callbacks: {
            dragEnd: onDragEnd
          }
        });

        // Must update Tagify's value according to the re-ordered nodes in the DOM.
        function onDragEnd(){
          tagify.updateValueByDOMTags()
        }

        function handleAutocomplete(value, selectedEntities){
          tagify.whitelist = null;
          // https://developer.mozilla.org/en-US/docs/Web/API/AbortController/abort
          controller && controller.abort();
          controller = new AbortController();
          // Show loading animation meanwhile the dropdown suggestions are hided.
          value !== '' ? tagify.loading(true) : tagify.loading(false);
          fetch($(input).attr('data-autocomplete-url') + '?q=' + encodeURIComponent(value) + '&selected=' + selectedEntities, {signal: controller.signal})
          .then(res => res.json())
          .then(function (newWhitelist) {
            let newWhitelistData = [];
            newWhitelist.forEach(function (current) {
              newWhitelistData.push({
                value: current.label,
                entity_id: current.entity_id,
                ...current.attributes
              });
            });
            if (newWhitelistData) {
              // build the whitelist with the values coming from the fetch
              tagify.whitelist = newWhitelistData; // update whitelist Array in-place
              tagify.loading(false).dropdown.show(value) // render the suggestions dropdown
            }
          }).catch(error => {
            console.error('Error fetching data:', error);
          });
        }

        // onInput event.
        let onInput = Drupal.debounce(function (e) {
          let value = e.detail.value;
          handleAutocomplete(value, tagify.value.map(item => item.entity_id));
        }, 500);

        // onEditInput event.
        let onEditInput = Drupal.debounce(function (e) {
          let value = e.detail.data.newValue;
          handleAutocomplete(value, tagify.value.map(item => item.entity_id));
        }, 500);

        // Edit input event (When user is editing the tag).
        tagify.on('edit:input', onEditInput)
        // Input event (When user is creating the tag).
        tagify.on('input', onInput)

        // If 'On click' dropdown suggestions is enabled.
        if (!tagify.settings.dropdown.enabled) {
          document.addEventListener('click', handleClickEvent);
        }

        function handleClickEvent(event) {
          const identifier = input.attributes.name.value;
          const containerClass = '.' + identifier;
          const isTagifyInput = event.target.classList.contains('tagify__input');
          const isDesiredContainer = event.target.closest(containerClass);
          if (isTagifyInput && isDesiredContainer) {
            handleAutocomplete('', tagify.value.map(item => item.entity_id));
          }
        }

      });
    }
  };
})(jQuery, Drupal, drupalSettings);
